@@ -31,14 +31,14 @@ sim_Rasch(rnorm(5),c(-.5,0,.8))
 sim_PV = function(theta, delta) {
   simulated_data = sim_Rasch(theta, delta) 
   parms = fit_enorm(simulated_data)
-  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"))
+  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"),bty='l')
   pv = plausible_values(simulated_data, parms, nPV = 10)
   for (i in 1:10) lines(ecdf(pv[[i+3]]-mean(pv[[i+3]])), col=rgb(.7,.7,.7,.5))
   lines(ecdf(theta-mean(theta)))
   invisible(NULL)
 }
 
-## ----fig.align='center', results='hide'----------------------------------
+## ----fig.align='center', results='hide',fig.height=4,fig.width=4---------
 theta = rnorm(300)
 delta = runif(50, -2, 1)
 sim_PV(theta, delta)
@@ -46,7 +46,7 @@ sim_PV(theta, delta)
 ## ----fig.align='center', results='hide', fig.width=7---------------------
 grp = sample(2, 300, replace = TRUE, prob = c(.5,.5))
 theta = rnorm(300, mean = c(-2,2)[grp], sd = c(1,1)[grp])
-plot(density(theta))
+plot(density(theta),bty='l',main='')
 par(mfrow=c(1,3))
 sim_PV(theta, delta[1:10])
 sim_PV(theta, delta[1:20])
@@ -57,7 +57,7 @@ par(mfrow=c(1,1))
 sim_PV2 = function(theta, delta) {
   simulated_data = sim_Rasch(theta, delta) 
   parms = fit_enorm(simulated_data, method="Bayes", nIterations = 50)
-  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"))
+  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"),bty='l')
   which.draw=5*(1:10)
   for (iter in 1:10) {
     pv = plausible_values(simulated_data, parms, use_draw=which.draw[iter])
@@ -88,7 +88,7 @@ sim_Rasch2 = function(theta, delta,group) {
 sim_PV3 = function(theta, delta, group) {
   simulated_data = sim_Rasch2(theta, delta,group) 
   parms = fit_enorm(simulated_data)
-  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"))
+  plot(ecdf(theta-mean(theta)), main=paste(length(delta), "items"),bty='l')
   pv = plausible_values(simulated_data, parms, covariates="group", nPV = 10)
   for (i in 1:10) lines(ecdf(pv[[i+3]]-mean(pv[[i+3]])), col=rgb(.7,.7,.7,.5))
   lines(ecdf(theta-mean(theta)))
@@ -99,5 +99,5 @@ par(mfrow=c(1,3))
 sim_PV3(theta, delta[1:10],grp)
 sim_PV3(theta, delta[1:20],grp)
 sim_PV3(theta, delta,grp)
-par(mfrow=c(1,1))
+
 
